from typing import Optional

from clipped.utils.versions import clean_version_for_check
from rest_framework import exceptions
from rest_framework.generics import GenericAPIView

from django.conf import settings
from django.http import HttpRequest, HttpResponse

from haupt.common import auditor, user_system
from haupt.common.apis.gzip import gzip
from haupt.common.apis.regex import OWNER_NAME_KEY, PROJECT_NAME_KEY, UUID_KEY
from haupt.common.authentication.bot import is_bot_user
from haupt.common.authentication.internal import is_internal_user
from haupt.common.endpoints import mixins
from haupt.common.endpoints.exceptions import EndpointError
from haupt.common.headers import get_service_header
from haupt.common.user_system import USER_SYSTEM_ID, USER_SYSTEM_NAME


class BaseEndpoint(
    mixins.CreateModelMixin,
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.DestroyModelMixin,
    mixins.UpdateModelMixin,
    GenericAPIView,
):
    serializer_class_mapping = None

    AUDITOR_EVENT_TYPES = None
    AUDIT_OWNER = False
    AUDIT_PROJECT = False
    AUDIT_PROJECT_RESOURCES = False
    AUDIT_INSTANCE = False
    AUDIT_EXTRA_KEYS = ()

    PROJECT_NAME_KEY = PROJECT_NAME_KEY
    PROJECT_OWNER_NAME_KEY = OWNER_NAME_KEY
    PROJECT_RESOURCE_KEY = None

    CONTEXT_KEYS = ()
    QUERY_CONTEXT_KEYS = ()
    CONTEXT_OBJECTS = ()

    ALLOWED_METHODS = ()
    ALLOWED_SERVICES = ()
    CHECK_SERVICE = False
    throttle_scope = "user"

    # This is a memoization for get_object, to avoid accidentally calling twice.
    _object = None
    _instance_id = None
    _instance_uuid = None
    _owner_id = None

    def set_owner(self):
        pass

    def get_object(self):
        """We memoize the access to this function in case a second call is made."""
        if self._object:
            return self._object
        self._object = super().get_object()
        return self._object

    def filter_queryset(self, queryset):
        queryset = self.enrich_queryset(queryset=queryset)
        return super().filter_queryset(queryset=queryset)

    def enrich_queryset(self, queryset):
        return queryset

    @staticmethod
    def get_service(request: HttpRequest) -> Optional[str]:
        service = get_service_header(request=request)
        if service:
            try:
                return service.decode()
            except UnicodeError:
                msg = (
                    "Invalid internal_service header. "
                    "internal_service string should not contain invalid characters."
                )
                raise exceptions.AuthenticationFailed(msg)

    @classmethod
    def get_service_version(cls, request: HttpRequest) -> Optional[str]:
        service = cls.get_service(request)
        if not service:
            return None
        service = clean_version_for_check(service)
        return service if len(service.split(".")) >= 3 else None

    @classmethod
    def check_service(cls, request: HttpRequest):
        service = cls.get_service(request=request)

        if (
            cls.CHECK_SERVICE
            and cls.ALLOWED_SERVICES
            and service not in cls.ALLOWED_SERVICES
        ):
            raise exceptions.AuthenticationFailed(
                "Service requesting this endpoint not allowed."
            )

    @classmethod
    def check_method(cls, request: HttpRequest):
        if request.method not in cls.ALLOWED_METHODS:
            raise exceptions.MethodNotAllowed(method=request.method)

    def check_request(self, request: HttpRequest):
        self.check_method(request)
        self.check_service(request)

    @staticmethod
    def check_public_only_access(request):
        return request.access.public_only

    def initialize_context(self, request: HttpRequest, *args, **kwargs):
        """
        Initializes the endpoint with the context keys based on the passed
        and/or based on the query parameters (request.GET).
        """
        self.check_request(request)

        for key in self.CONTEXT_KEYS:
            if key not in kwargs:
                raise EndpointError(
                    f"Endpoint `{self.__class__.__name__}` requires a context key {key}, "
                    f"but it was not found."
                )
            setattr(self, key, kwargs.get(key))
        for key in self.QUERY_CONTEXT_KEYS:
            setattr(self, key, request.query_params.get(key))

        if self.AUDIT_OWNER:
            assert OWNER_NAME_KEY in kwargs
        if self.AUDIT_PROJECT:
            assert PROJECT_NAME_KEY in kwargs
        if self.AUDIT_PROJECT_RESOURCES:
            assert self.PROJECT_RESOURCE_KEY is not None
            assert self.PROJECT_RESOURCE_KEY in kwargs

    def initialize_object_context(self, request: HttpRequest, *args, **kwargs):
        pass

    def initial(self, request, *args, **kwargs):
        super().initial(request, *args, **kwargs)
        self.initialize_context(request, *args, **kwargs)
        self.initialize_object_context(request, *args, **kwargs)

    def initialize_request(self, request, *args, **kwargs):
        request = super().initialize_request(request, *args, **kwargs)
        request.access = None
        return request

    def get_audit_params(self, **kwargs):
        params = {}
        if self.AUDIT_OWNER:
            params[self.PROJECT_OWNER_NAME_KEY] = kwargs.get(OWNER_NAME_KEY)
        if self.AUDIT_PROJECT:
            params[self.PROJECT_NAME_KEY] = kwargs.get(PROJECT_NAME_KEY)
        if self.AUDIT_PROJECT_RESOURCES:
            params[UUID_KEY] = kwargs.get(self.PROJECT_RESOURCE_KEY)
        if self.AUDIT_INSTANCE:
            params["instance"] = self._object
        for extra_key in self.AUDIT_EXTRA_KEYS:
            params[extra_key] = kwargs.get(extra_key)
        return params

    def get_event_type(self, request) -> Optional[str]:
        event_types = self.AUDITOR_EVENT_TYPES or {}
        event_type = event_types.get(request.method)
        if not settings.HAS_ORG_MANAGEMENT:
            return event_type
        if (
            event_type
            and not user_system.is_system_user(request.user.id)
            and not request.user.is_anonymous
            and not is_bot_user(request.user)
            and not is_internal_user(request.user)
        ):
            return event_type

    def audit_check(self, request):
        return True

    def audit(self, request, *args, **kwargs):
        event_type = self.get_event_type(request)
        if event_type and self.audit_check(request):
            self.set_owner()
            params = self.get_audit_params(**kwargs)
            auditor.record(
                event_type=event_type,
                actor_id=self.request.user.id
                if settings.HAS_ORG_MANAGEMENT
                else USER_SYSTEM_ID,
                actor_name=self.request.user.username
                if settings.HAS_ORG_MANAGEMENT
                else USER_SYSTEM_NAME,
                owner_id=self._owner_id,
                **params,
            )

    def get_serializer_class(self):
        """
        Return the class to use for the serializer.
        Defaults to using `self.serializer_class`.

        You may want to override this if you need to provide different
        serializations depending on the incoming request.

        (Eg. admins get full serialization, others get basic serialization)
        """
        assert (
            self.serializer_class is not None
            or self.serializer_class_mapping is not None
        ), (
            "'%s' should either include a `serializer_class` attribute or "
            "a `serializer_class_mapping` attribute, "
            "or override the `get_serializer_class()` method." % self.__class__.__name__
        )

        if self.serializer_class_mapping:
            return self.serializer_class_mapping[self.request.method]
        return self.serializer_class


class CreateEndpoint:
    def post(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        return self.create(request, *args, **kwargs)


class PostEndpoint:
    def post(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        return self.create(request, *args, **kwargs)

    def get_serializer(self, *args, **kwargs):
        pass


class ListEndpoint:
    @gzip()
    def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        return self.list(request, *args, **kwargs)


class RetrieveEndpoint:
    def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        return self.retrieve(request, *args, **kwargs)


class DestroyEndpoint:
    def delete(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        return self.destroy(request, *args, **kwargs)


class UpdateEndpoint:
    def put(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        return self.update(request, *args, **kwargs)

    def patch(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        return self.partial_update(request, *args, **kwargs)



from haupt.common.options.option import Option, OptionScope, OptionStores

POLYAXON_ENVIRONMENT = "POLYAXON_ENVIRONMENT"
PLATFORM_VERSION = "PLATFORM_VERSION"
PLATFORM_DIST = "PLATFORM_DIST"
PLATFORM_MODE = "PLATFORM_MODE"
PLATFORM_HOST = "PLATFORM_HOST"
CHART_VERSION = "CHART_VERSION"
ORGANIZATION_ID = "POLYAXON_ORGANIZATION_ID"
ORGANIZATION_NAME = "POLYAXON_ORGANIZATION_NAME"
ORGANIZATION_PLAN = "POLYAXON_ORGANIZATION_PLAN"
ORGANIZATION_LICENSE = "POLYAXON_ORGANIZATION_LICENSE"
ORGANIZATION_HMAC = "POLYAXON_ORGANIZATION_HMAC"
ORGANIZATION_KEY = "POLYAXON_ORGANIZATION_KEY"

OPTIONS = {
    POLYAXON_ENVIRONMENT,
    PLATFORM_VERSION,
    PLATFORM_DIST,
    PLATFORM_MODE,
    PLATFORM_HOST,
    CHART_VERSION,
    ORGANIZATION_ID,
    ORGANIZATION_NAME,
    ORGANIZATION_PLAN,
    ORGANIZATION_LICENSE,
    ORGANIZATION_HMAC,
    ORGANIZATION_KEY,
}


class PlatformEnvironmentVersion(Option):
    key = POLYAXON_ENVIRONMENT
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.SETTINGS
    default = None
    options = None


class PlatformVersion(Option):
    key = PLATFORM_VERSION
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.SETTINGS
    default = None
    options = None


class PlatformDist(Option):
    key = PLATFORM_DIST
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.SETTINGS
    default = None
    options = None


class PlatformMode(Option):
    key = PLATFORM_MODE
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.SETTINGS
    default = None
    options = None


class PlatformHost(Option):
    key = PLATFORM_HOST
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "str"
    default = None
    options = None


class ChartVersion(Option):
    key = CHART_VERSION
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.SETTINGS
    default = None
    options = None


class OrganizationId(Option):
    key = ORGANIZATION_ID
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.ENV
    default = None
    options = None


class OrganizationName(Option):
    key = ORGANIZATION_NAME
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.ENV
    default = None
    options = None


class OrganizationPlan(Option):
    key = ORGANIZATION_PLAN
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "dict"
    store = OptionStores.ENV
    default = None
    options = None


class OrganizationLicense(Option):
    key = ORGANIZATION_LICENSE
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.ENV
    default = None
    options = None


class OrganizationHmac(Option):
    key = ORGANIZATION_HMAC
    scope = OptionScope.GLOBAL
    is_secret = True
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.ENV
    default = None
    options = None


class OrganizationKey(Option):
    key = ORGANIZATION_KEY
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.ENV
    default = None
    options = None



from rest_framework import serializers

from polyaxon.schemas import V1RunResources


class ResourcesMixin(serializers.Serializer):
    def get_resources(self, obj):
        return {
            V1RunResources._MEMORY: obj.memory,
            V1RunResources._CPU: obj.cpu,
            V1RunResources._GPU: obj.gpu,
            V1RunResources._CUSTOM_RESOURCE: obj.custom,
            V1RunResources._COST: obj.cost,
        }



class CeleryQueues:
    """Celery Queues.

    N.B. make sure that the queue name is not < 128.
    """

    SCHEDULER_HEALTH = "queues.scheduler.health"
    SCHEDULER_RUNS = "queues.scheduler.runs"
    SCHEDULER_COMPILER = "queues.scheduler.compiler"
    SCHEDULER_ARTIFACTS = "queues.scheduler.artifacts"
    SCHEDULER_CLEAN = "queues.scheduler.clean"

    @classmethod
    def scheduler(cls):
        return f"{cls.SCHEDULER_HEALTH},{cls.SCHEDULER_COMPILER},{cls.SCHEDULER_RUNS},{cls.SCHEDULER_ARTIFACTS},{cls.SCHEDULER_CLEAN}"

    @classmethod
    def all(cls):
        return cls.scheduler()



import logging

from typing import Dict

from clipped.utils.bools import to_bool

from django.contrib.auth import get_user_model
from django.contrib.auth.password_validation import validate_password
from django.core import exceptions
from django.core.exceptions import ValidationError
from django.core.management.base import BaseCommand, CommandError

_logger = logging.getLogger("haupt.commands")


class Command(BaseCommand):
    """Management utility to create users/superusers.

    This is command is different from the django one, because:
        1. does not prompt the user to enter a password, i.e. can be used inline.
        2. validates and requires an email.
    """

    help = "Used to create a user/superuser."
    requires_migrations_checks = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.UserModel = get_user_model()
        # pylint:disable= protected-access
        self.username_field = self.UserModel._meta.get_field(
            self.UserModel.USERNAME_FIELD
        )
        # pylint:disable= protected-access
        self.email_field = self.UserModel._meta.get_field("email")

    def add_arguments(self, parser):
        parser.add_argument(
            "--%s" % self.UserModel.USERNAME_FIELD,
            required=True,
            dest=self.UserModel.USERNAME_FIELD,
            help="Specifies the login for the user/superuser.",
        )
        parser.add_argument(
            "--password",
            required=True,
            dest="password",
            help="Specifies the password for the user/superuser.",
        )
        parser.add_argument(
            "--email",
            required=True,
            dest="email",
            help="Specifies the email for the user/superuser.",
        )
        parser.add_argument(
            "--superuser",
            dest="is_superuser",
            action="store_true",
            default=False,
            help="Specifies a user or superuser.",
        )
        parser.add_argument(
            "--force",
            dest="force",
            action="store_true",
            default=False,
            help="To force create the user even if the user is not valid.",
        )

    def validate_password(self, password: str, user_data: Dict, force: bool):
        try:
            validate_password(password, self.UserModel(**user_data))
        except ValidationError as e:
            _logger.warning("The password provided is not valid %s", e)
            if force:
                _logger.warning(
                    "The user will be created although the password does not meet the validation."
                )
            else:
                raise e

    def handle(self, *args, **options):  # pylint:disable=too-many-branches
        username = options[self.UserModel.USERNAME_FIELD].strip()
        password = options["password"].strip()
        email = options["email"].strip()
        force = to_bool(options["force"])
        is_superuser = to_bool(options["is_superuser"])

        try:
            username = self.username_field.clean(username, None)
        except exceptions.ValidationError as e:
            raise CommandError("; ".join(e.messages))

        try:
            self.email_field.clean(email, None)
        except exceptions.ValidationError as e:
            raise CommandError("; ".join(e.messages))

        try:
            self.UserModel.objects.get_by_natural_key(username)
        except self.UserModel.DoesNotExist:
            pass
        else:
            _logger.info(
                "Info: Username %s is already taken. Will not recreate user.", username
            )
            return

        try:
            self.UserModel.objects.get(email=email)
        except self.UserModel.DoesNotExist:
            pass
        except exceptions.MultipleObjectsReturned:
            raise CommandError("Error: That %s is already taken." % email)
        else:
            raise CommandError("Error: That %s is already taken." % email)

        if not username:
            raise CommandError("Error: Blank username aren't allowed.")

        if not password:
            raise CommandError("Error: Blank passwords aren't allowed.")

        if not email:
            raise CommandError("Error: Blank email aren't allowed.")

        user_data = {self.UserModel.USERNAME_FIELD: username, "email": email}

        self.validate_password(password=password, user_data=user_data, force=force)
        user_data["password"] = password

        if is_superuser:
            self.UserModel.objects.create_superuser(**user_data)
        else:
            self.UserModel.objects.create_user(**user_data)

        if options["verbosity"] >= 1:
            self.stdout.write(
                "{} created successfully.".format(
                    "Superuser" if is_superuser else "User"
                )
            )



from haupt.common.apis.regex import OWNER_NAME_PATTERN, RUN_UUID_PATTERN

URLS_CATALOGS_AGENTS_STATE = r"^orgs/{}/agents/state/?$".format(OWNER_NAME_PATTERN)
URLS_CATALOGS_AGENTS_CRON = r"^orgs/{}/agents/cron/?$".format(OWNER_NAME_PATTERN)
URLS_ORGANIZATIONS_RUNS_DETAILS = r"^orgs/{}/runs/{}/?$".format(
    OWNER_NAME_PATTERN, RUN_UUID_PATTERN
)



from haupt.schemas.platform_config import PlatformConfig


def set_admin(context, config: PlatformConfig):
    if config.admin_mail and config.admin_mail:
        admins = ((config.admin_name, config.admin_mail),)
        context["ADMINS"] = admins
        context["MANAGERS"] = admins



from haupt.proxies.schemas.base import get_config

HEALTHZ_LOCATION_OPTIONS = r"""
location /healthz/ {{
    access_log off;
    return 200 "healthy";
}}
"""


def get_healthz_location_config():
    return get_config(
        options=HEALTHZ_LOCATION_OPTIONS,
        indent=0,
    )



UUID_KEY = "uuid"
OWNER_KEY = "uuid"
NAME_KEY = "name"
USERNAME_KEY = "username"
VERSION_KEY = "version"
VERSION_PATTERN = r"(?P<version>[-\w]{1,16})"
INSTALLATION_KEY = "installation"
INSTALLATION_PATTERN = r"(?P<installation>\w{1,36})"
UUID_PATTERN = r"(?P<uuid>\w{1,36})"
SLUG_PATTERN = r"[-\w]{1,256}"
USERNAME_PATTERN = r"(?P<username>[-\w]{1,128})"
OWNER_NAME_KEY = "owner_name"
PROJECT_OWNER_NAME_KEY = "project_owner_name"
OWNER_NAME_PATTERN = r"(?P<owner_name>[-\w]{1,128})"
PROJECT_NAME_KEY = "project_name"
PROJECT_NAME_PATTERN = r"(?P<project_name>[-\w]{1,128})"
ARTIFACT_NAME_PATTERN = r"(?P<artifact_name>[-\w]{1,128})"
ARTIFACT_NAME_KEY = "artifact_name"
NAME_PATTERN = r"(?P<name>[-\w]{1,256})"
SEQUENCE_PATTERN = r"(?P<sequence>\d+)"
ID_PATTERN = r"(?P<id>\d+)"
INDEX_PATTERN = r"(?P<index>\d+)"
RUN_ID_PATTERN = r"(?P<run_id>\d+)"
RUN_UUID_KEY = "run_uuid"
RUN_UUID_PATTERN = r"(?P<run_uuid>\w{1,36})"
RUN_SVC_PATTERN = r"(?P<svc_url>[-\w]{1,256})"
RUN_PORT_PATTERN = r"(?P<artifact_name>[-\w]{1,128})"

EVENT_KIND_PATTERN = r"(?P<event_kind>[-\w]{1,20})"
EVENT_NAME_PATTERN = r"(?P<event_name>[-\w]{1,128})"

ANY_CONSUMER_PATTERN = r"?(?P<path>.*)/?"

VERSION_NAME_KEY = "version_name"
VERSION_NAME_PATTERN = r"(?P<version_name>[-.\w]{1,128})"
TEAM_NAME_KEY = "team_name"
TEAM_NAME_PATTERN = r"(?P<team_name>[-\w]{1,256})"
INTEGRATION_UUID_KEY = "integration_uuid"
INTEGRATION_UUID_PATTERN = r"(?P<integration_uuid>\w{1,36})"
PROVIDER_KEY = "provider"
PROVIDER_PATTERN = r"(?P<provider>\w{0,50})"



from haupt.proxies.generators.api import generate_api_conf
from haupt.proxies.generators.forward import generate_forward_proxy_cmd
from haupt.proxies.generators.gateway import generate_gateway_conf
from haupt.proxies.generators.streams import generate_streams_conf



import uuid

from polyaxon._constants.globals import DEFAULT


class Owner:
    name = DEFAULT
    uuid = uuid.UUID("9b0a3806e3f84ea1959a7842e34129ed")
    id = 1


class Actor:
    username = DEFAULT
    id = 1



import logging

from typing import Dict, Optional

from clipped.utils.bools import to_bool
from clipped.utils.dates import parse_datetime
from clipped.utils.serialization import datetime_serialize
from rest_framework import status

from django.core.handlers.asgi import ASGIRequest
from django.db import transaction
from django.http import HttpResponse
from django.urls import path

from haupt.common.endpoints.validation import validate_internal_auth, validate_methods
from haupt.streams.connections.fs import AppFS
from haupt.streams.controllers.k8s_crd import get_k8s_operation
from haupt.streams.controllers.logs import (
    get_archived_operation_logs,
    get_operation_logs,
    get_tmp_operation_logs,
)
from haupt.streams.endpoints.base import UJSONResponse
from haupt.streams.tasks.logs import clean_tmp_logs, upload_logs
from haupt.streams.tasks.op_spec import upload_op_spec
from polyaxon import settings
from polyaxon._k8s.logging.async_monitor import get_op_spec, query_k8s_operation_logs
from polyaxon._k8s.manager.async_manager import AsyncK8sManager
from polyaxon._utils.fqn_utils import get_resource_name, get_resource_name_for_kind

logger = logging.getLogger("haupt.streams.logs")


@transaction.non_atomic_requests
async def get_run_logs(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    methods: Optional[Dict] = None,
) -> UJSONResponse:
    validate_methods(request, methods)
    force = to_bool(request.GET.get("force"), handle_none=True)
    connection = request.GET.get("connection")
    last_time = request.GET.get("last_time")
    if last_time:
        last_time = parse_datetime(last_time).astimezone()
    last_file = request.GET.get("last_file")
    files = []

    if last_time:
        resource_name = get_resource_name(run_uuid=run_uuid)

        k8s_manager = AsyncK8sManager(
            namespace=namespace,
            in_cluster=settings.CLIENT_CONFIG.in_cluster,
        )
        await k8s_manager.setup()
        k8s_operation = await get_k8s_operation(
            k8s_manager=k8s_manager, resource_name=resource_name
        )
        if k8s_operation:
            operation_logs, last_time = await get_operation_logs(
                k8s_manager=k8s_manager,
                k8s_operation=k8s_operation,
                instance=run_uuid,
                last_time=last_time,
            )
        else:
            operation_logs, last_time = await get_tmp_operation_logs(
                fs=await AppFS.get_fs(connection=connection),
                store_path=AppFS.get_fs_root_path(connection=connection),
                run_uuid=run_uuid,
                last_time=last_time,
            )
        if k8s_manager:
            await k8s_manager.close()

    else:
        operation_logs, last_file, files = await get_archived_operation_logs(
            fs=await AppFS.get_fs(connection=connection),
            store_path=AppFS.get_fs_root_path(connection=connection),
            run_uuid=run_uuid,
            last_file=last_file,
            check_cache=not force,
        )
    data = dict(
        last_time=datetime_serialize(last_time),
        last_file=last_file,
        logs=operation_logs,
        files=files,
    )
    return UJSONResponse(data)


@transaction.non_atomic_requests
async def collect_run_logs(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    run_kind: str,
    methods: Optional[Dict] = None,
) -> HttpResponse:
    validate_methods(request, methods)
    try:
        validate_internal_auth(request)
    except Exception as e:
        errors = "Request requires an authenticated internal service %s" % e
        logger.warning(errors)
        return UJSONResponse(
            data={"errors": errors},
            status=status.HTTP_400_BAD_REQUEST,
        )
    resource_name = get_resource_name_for_kind(run_uuid=run_uuid, run_kind=run_kind)
    k8s_manager = AsyncK8sManager(
        namespace=namespace,
        in_cluster=settings.CLIENT_CONFIG.in_cluster,
    )
    await k8s_manager.setup()
    k8s_operation = await get_k8s_operation(
        k8s_manager=k8s_manager, resource_name=resource_name
    )
    if not k8s_operation:
        errors = (
            "Run's logs were not collected, resource was not found for run %s"
            % run_uuid
        )
        logger.warning(errors)
        return UJSONResponse(
            data={"errors": errors},
            status=status.HTTP_400_BAD_REQUEST,
        )
    operation_logs, _ = await query_k8s_operation_logs(
        instance=run_uuid, k8s_manager=k8s_manager, last_time=None
    )
    op_spec, _, _ = await get_op_spec(
        k8s_manager=k8s_manager, run_uuid=run_uuid, run_kind=run_kind
    )
    if k8s_manager:
        await k8s_manager.close()
    if not operation_logs:
        # TODO: Add logic to handle job without logs
        errors = "Operation logs could not be fetched for run %s" % run_uuid
        logger.warning(errors)
        return HttpResponse(
            content={"errors": errors},
            status=status.HTTP_404_NOT_FOUND,
        )

    fs = await AppFS.get_fs()
    store_path = AppFS.get_fs_root_path()
    try:
        await upload_logs(
            fs=fs, store_path=store_path, run_uuid=run_uuid, logs=operation_logs
        )
    except Exception as e:
        errors = (
            "Run's logs was not collected, an error was raised while uploading the data. "
            "Error %s." % e
        )
        logger.warning(errors)
        return UJSONResponse(
            data={"errors": errors},
            status=status.HTTP_400_BAD_REQUEST,
        )

    try:
        await clean_tmp_logs(fs=fs, store_path=store_path, run_uuid=run_uuid)
    except Exception as e:
        errors = "Did not remove temp logs, an error was raised. " "Error %s." % e
        logger.debug(errors)

    if op_spec:
        try:
            await upload_op_spec(
                fs=fs,
                store_path=store_path,
                run_uuid=run_uuid,
                op_spec=op_spec,
            )
        except Exception as e:
            errors = (
                "Operation spec was not collected, an error was raised while uploading the data. "
                "Error %s." % e
            )
            logger.warning(errors)

    return HttpResponse(status=status.HTTP_200_OK)


URLS_RUNS_COLLECT_LOGS = (
    "<str:namespace>/<str:owner>/<str:project>/runs/<str:run_uuid>/<str:run_kind>/logs"
)
URLS_RUNS_LOGS = "<str:namespace>/<str:owner>/<str:project>/runs/<str:run_uuid>/logs"


# fmt: off
logs_routes = [
    path(
        URLS_RUNS_LOGS,
        get_run_logs,
        name="get_run_logs",
        kwargs=dict(methods=["GET"]),
    ),
]
internal_logs_routes = [
    path(
        URLS_RUNS_COLLECT_LOGS,
        collect_run_logs,
        name="collect_run_logs",
        kwargs=dict(methods=["POST"]),
    ),
]
